/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#include <algorithm>    // std::max
using namespace std;

#include "seamless3d.h"

extern bool g_outputPovRay;
extern Ptr<PartCageClue> g_partCageClue;
extern Coordinate* g_nurbsIlsCoord;
extern ScenePtr g_scene;
extern IndexedLineSet* g_nurbsIls;
extern ArtPtr g_vcNurbsCtrlPolyPart;
extern Color* g_nurbsIlsColor;
extern Matrix g_viewMatrix;
extern M_Vec2f g_screenPoint;
extern M_Bool g_pointIsVisible;
extern Ptr<TransformCP> g_transformCP;
extern M_Int g_pointLookup;
extern TransformPtr g_vcBezierBase;
extern ShapePtr g_nurbsIlsShape;
extern bool g_showPatchBorderMode;
extern InsertPatch g_insertPatch;

int getCorner(int corner, int offset, int len);
void calcMidPoints(const int i, const M_CtrlPolyPoint& ctrlPolyPoint);
void calcPolyCenterScreenPoints(const M_CtrlPolyPoint& ctrlPolyPoint, M_Vec2f& m_screenPoint, M_Bool& m_pointIsVisible, const Matrix& viewMatrix);
void addRelevantPart(M_Art& relevant, Art*& part);
bool partAllreadyIsRelevant(M_Art& relevant, Art*& part);
void encodeStitchIndex(M_Int& stitchIndex, const M_CtrlPolyPoint& ctrlPolyPoint);
Rotation4d convRot4fto4d(const Rotation& par);
void genHAnimSkinCoordWeight(Art* parentPart, HAnimJointPtr parentJoint, int& skinCoordIndexCntr, const bool usingPivotPoints);
void changeFloatFormat(int numOfDigits);
String addTransformScript(Seamless* smls, M_Int& partTree);
void addMorpherToScript(Script*& script, M_Properties& properties, const Art*& part, Morpher* morpher, String& str);
void addEffect(M_<Effect*>& effectList, Effect* effect);
void creatPartLookUp(M_Art& partLookUp, M_Art& partList);
void processAllPatches(M_<NurbsPatch*>& cpNodeList);

Node* Seamless::EffectField::operator = (Node* par){
    refValue()=dynamic_cast<Effect*>(par);
    home->setUpInternalEffectPtrs();
    if(par==0)return 0;
    home->process();
    M_Art& partList = home->partList;
    home->effect->updateNewEffect(partList);
    return getRawPtr();
}
void Seamless::SurfaceNormals::operator=(const bool& par){
    refValue() = par;
}
void Seamless::BuildNormals::operator=(const bool& par){
    refValue() = par;
}


bool allRelevantsAreInferiour(Art* part){

    M_Art& relevantPart=part->relevantPart;

    for(int k=relevantPart.length; --k>=0;){
        if(relevantPart[k]->plidx>part->plidx){
            return false;
        }
    }
    return true;
}//return true if part belongs to  relevant list
bool weBelong(M_Art& relevantPart, Art* part){

    for(int k=relevantPart.length; --k>=0;){
        if(relevantPart[k]==part){
            return true;
        }
    }
    return false;
}
void verifyAllVerticesAreReferenced(M_Art& partList){
    int count=0;
    int k;
    for(k=partList.length; --k>=0; ){
        Art* part=partList[k];
        part->partTree.length=0;	//set all elements to 0;
        part->partTree.length=part->coord.length;
    }
    for(k=partList.length; --k>=0; ){
        Art* part=partList[k];
        M_Triangle& triangle=part->triangle;
        for(int i=triangle.length*3; --i>=0;){
            triangle.refPart(i)->partTree[triangle.refIndex(i)]=1;
        }
    }
    for(k=partList.length; --k>=0; ){
        Art* part=partList[k];
        for(int i=part->coord.length; --i>=0;){
            if(part->partTree[i]==0){
                count++;
                dLog(part->name+String(" ")+i);
            }
        }
    }
    if(count){
        dLog(String("unreferenced texCoord count = ")+count);
    }
}
bool verifyTrianglesDontRefDown(Art* part){
    int plidx=part->plidx;
    M_Triangle& triangle=part->triangle;
    for(int i=triangle.length; --i>=0; ){
        S_Triangle tri=triangle[i];
        if(	tri.pa->plidx<plidx || tri.pb->plidx<plidx || tri.pc->plidx<plidx){
            return false;
        }
    }
    return true;
}
bool verifyAllTrianglesDontRefUp(Seamless* smls){
    for(int k=smls->partList.length; --k>=0; ){
        if(verifyTrianglesDontRefDown(smls->partList[k])==false){
            throw smls->partList[k];
        }
    }
    return true;
}
int numOfConsecutivePoints(const M_ControlPoint& controlPoint, int& g, NurbsPatch* cpNode, int idx, int controlPointLen){
    int numOf = 1;
    while(g < controlPointLen && controlPoint[g].p == cpNode && controlPoint[g].i == idx){
        g++;
        numOf++;
        idx++;
    }
    return numOf;
}
void Seamless::linkToString(String& str){
    if(link.length == 0){
        return;
    }
    str+="\nlink[";

    for(int i = 0; ;){
        str+=link[i].patch->fromPoint + link[i].point;
        str+=' ';
        str+=link[i].poly;
        str+=' ';
        str+=link[i].corner;
        str+=' ';
        str+=link[i].between;
        if(++i >= link.length){
            break;
        }
        str+=' ';
    }
    str+=']';
}
void Seamless::readLink(const String& str, int& i){
    str.stb(i);
    link.length=0;
    if(str[i]=='['){
        i++;
        str.stb(i);
        int k = i;
        int ctr = 0;

        for(;str[k]!=']';ctr++){
            getInt(str, k);
            str.stb(k);
        }
        link.length = ctr / 4;

        for(ctr=0; str[i] != ']'; ctr++){
            int point = getInt(str, i);
            str.stb(i);
            int poly = getInt(str, i);
            str.stb(i);
            int corner = getInt(str, i);
            str.stb(i);
            float between = getFloat(str, i);
            str.stb(i);
            link[ctr] = Link(point, poly, corner, between);
        }
        i++;
    }
}
void Seamless::resetRelevantParts(){
    int k;
    for( k=partList.length; --k>=0; ){
        partList[k]->relevantPart.length=0;
    }
    for( k=partList.length; --k>=0; ){
        Art* part=partList[k];
        M_Triangle triangle=part->triangle;
        for (int i=triangle.length; --i>=0;){
            S_Triangle tri=triangle[i];
            addRelevantPart(tri.pa->relevantPart, part);
            addRelevantPart(tri.pb->relevantPart, part);
            addRelevantPart(tri.pc->relevantPart, part);
        }
    }
}
bool Seamless::verifyAllRelevantPartsExist(){//does not check if any are irelevant
    for(int k = partList.length; --k>=0; ){
        Art* part=partList[k];
        M_Triangle triangle=part->triangle;
        for (int i=triangle.length; --i>=0;){
            S_Triangle tri=triangle[i];
            if(partAllreadyIsRelevant(tri.pa->relevantPart, part)==false) return false;
            if(partAllreadyIsRelevant(tri.pb->relevantPart, part)==false) return false;
            if(partAllreadyIsRelevant(tri.pc->relevantPart, part)==false) return false;
        }
    }
    return true;
}

void Seamless::verifyIntegrity(){//Fact: All inferior parts have no relivance (ilogical for them to own triangles that reference up)

    for(int k=0; k < partList.length; k++){
        Art* part = partList[k];
        M_Triangle& triangle = part->triangle;
        if(verifyAllRelevantPartsExist()==false){
//			_asm{int 3};
        }
//		if (allRelevantsAreReferenced(part)==false){
//			_asm{int 3};
//		}
        if(allRelevantsAreInferiour(part)==false){
            String tmpStr;
            part->triangle.toString(tmpStr);
            dLog(tmpStr);
            throw;
        }
        if(verifyTrianglesDontRefDown(part)==false){
            throw;
        }
        for(int i = 0; i < triangle.length; i++){
            S_Triangle tri = triangle[i];
            if( tri.pa!=part && tri.pb!=part && tri.pc!=part ){
                throw;
            }
            if(tri.ia<0 ){
                throw;
            }
            if(tri.ia>=tri.pa->coord.length+tri.pa->rc.length){
                throw;
            }
            if(tri.pa->plidx<part->plidx){
                throw;
            }
            if(weBelong(tri.pa->relevantPart, part)==false){
                throw;//part should be one of the superiours in pa's relevant list because pa's vertex is relevant to part's triangles
            }
            if(tri.ib<0 ){
                throw;
            }
            if(tri.ib>=tri.pb->coord.length+tri.pb->rc.length){
                throw;
            }
            if(tri.pb->plidx<part->plidx){
                throw;
            }
            if(weBelong(tri.pb->relevantPart, part)==false){
                throw;
            }
            if(tri.ic<0 ){
                throw;
            }
            if(tri.ic>=tri.pc->coord.length+tri.pc->rc.length){
                throw;
            }
            if(tri.pc->plidx<part->plidx){
                throw;
            }
            if(weBelong(tri.pc->relevantPart, part)==false){
                throw;
            }
        }
    }
    verifyAllVerticesAreReferenced(partList);
    return;
}
void Seamless::removeBuildNodes(M_Node& par1){
    int i,j,k;
    M_Build& build=this->build.refValue();
    for(j=0;j<par1.length;j++){
        for(i=0;i<build.length;i++){
            if(build[i]==par1[j]){
                /*
                Node* tmp=&*children[i];
                if(--tmp->refCount==0){
                    delete tmp;
                }
*/
                for(k=i;k<build.length-1;k++){
                    build[k]=build[k+1];
                }
                build.length--;
            }
        }
    }
}
void Seamless::convertToRc(int buildLength){
    if(partList.length==0 || partList[0]->workEffect == 0) return;

    if(partList[0]->workEffect->getType()==COLOREFFECT){
        ColorEffect* effect=dynamic_cast<ColorEffect*>(partList[0]->workEffect);
        if(effect==0) return;
        const M_Color3f& color=effect->color;
        static M_Int lookUpCoord;
        lookUpCoord.length = color.length;
        {
            for(int i=lookUpCoord.length; --i>=0; ){
                lookUpCoord[i]=-1;
            }
        }
        {
            int coordLen=0;
            int coordIndexLen=0;
            for(int k=0; k<partList.length; k++){
                Art* part=partList[k];
                if(buildLength > 0 && dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
                int clen = part->coord.length;
                part->color.length = clen;
                part->rc.length = 0;
                part->fromCoord = coordLen;
                M_Int& localCoordLookUp = part->partTree;
                localCoordLookUp.length = clen;
                coordLen+=clen;
                coordIndexLen+=part->triangle.length;
                for(int i=clen; --i>=0; ){
                    localCoordLookUp[i]=-1;
                }
            }
            coordIndexLen*=3;
            coordIndex.length=coordIndexLen;
        }
        {
            for(int j=0, k=0; k<partList.length; k++){
                Art* part=partList[k];
                if(dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
                M_Triangle& triangle=part->triangle;
                int triLen=triangle.length*3;
                for(int i=0; i<triLen; i++){
                    int linTci=triangle.refTexIndex(i);
                    int ci=triangle.refIndex(i);
                    Art* cornerPart=triangle.refPart(i);
                    M_Int& localCoordLookUp=cornerPart->partTree;
                    if( lookUpCoord[linTci]==-1){
                        if( localCoordLookUp[ci]==-1){
                            localCoordLookUp[ci]=linTci;
                            cornerPart->color[ci]=color[linTci];
                            lookUpCoord[linTci]=ci;
                            coordIndex[j++]=ci;
                        }
                        else {
                            coordIndex[j++]=lookUpCoord[linTci]=cornerPart->color.length;
                            cornerPart->color+=color[linTci];
                            cornerPart->rc+=ci;
                        }
                    }
                    else {
                        coordIndex[j++]=lookUpCoord[linTci];
                    }
                }
            }
        }
    }
    else if(partList[0]->workEffect->getType()==TEXTUREEFFECT){
        TextureEffect* effect=dynamic_cast<TextureEffect*>(partList[0]->workEffect);
        if(effect==0) return;
        M_Vec2f& texCoord=effect->texCoord.refValue();
        static M_Int lookUpCoord;
        lookUpCoord.length=texCoord.length;
        {
            for(int i=lookUpCoord.length; --i>=0; ){
                lookUpCoord[i]=-1;
            }
        }
        {
            int coordLen=0;
            int coordIndexLen=0;
            for(int k=0; k<partList.length; k++){
                Art* part=partList[k];
                if(dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
                int clen=part->coord.length;
                part->texCoord.length=clen;
                part->rc.length=0;
                part->fromCoord=coordLen;
                M_Int& localCoordLookUp=part->partTree;
                localCoordLookUp.length=clen;
                coordLen+=clen;
                coordIndexLen+=part->triangle.length;
                for(int i=clen; --i>=0; ){
                    localCoordLookUp[i]=-1;
                }
            }
            coordIndexLen*=3;
            coordIndex.length=coordIndexLen;
        }
        {
            for(int j=0, k=0; k<partList.length; k++){
                Art* part=partList[k];
                if(dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
                M_Triangle& triangle=part->triangle;
                int triLen=triangle.length*3;
                for(int i = 0; i<triLen; i++){
                    int linTci=triangle.refTexIndex(i);
                    int ci=triangle.refIndex(i);
                    Art* cornerPart=triangle.refPart(i);
                    M_Int& localCoordLookUp=cornerPart->partTree;
                    if( lookUpCoord[linTci]==-1){
                        if( localCoordLookUp[ci]==-1){
                            localCoordLookUp[ci]=linTci;
                            cornerPart->texCoord[ci]=texCoord[linTci];
                            lookUpCoord[linTci]=ci;
                            coordIndex[j++]=ci;
                        }
                        else {
                            coordIndex[j++]=lookUpCoord[linTci]=cornerPart->texCoord.length;
                            cornerPart->texCoord+=texCoord[linTci];
                            cornerPart->rc+=ci;
                        }
                    }
                    else {
                        coordIndex[j++]=lookUpCoord[linTci];
                    }
                }
            }
        }
    }
    {
        int linCoordLen=0;
        for(int k=0; k<partList.length; k++){
            Art* part=partList[k];
            if(dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
            part->fromCoord=linCoordLen;
            linCoordLen+=part->coord.length+part->rc.length; //part->texCoord.length;
        }
    }
    {
        for(int j=0, k=0; k<partList.length; k++){
            Art* part=partList[k];
            if(dynamic_cast<ImmediatePart*>(part) == 0 && bnspe == false)continue;
            M_Triangle& triangle=part->triangle;
            int triLen=triangle.length*3;
            {
                for(int i=0; i<triLen; ){
                    coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
                    coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
                    coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
                }
            }
            /*
            {
                M_RedirectCoord& rc=part->rc;
                for(int i=rc.length; --i>=0; ){
                    rc[i].i+=rc[i].p->fromCoord;
                }
            }
            */
        }
    }
}
void Seamless::encodePatchOwner(){
    M_Int& smlsPatchOwner = this->patchOwner;
    smlsPatchOwner.length = 0;
    for(int k = 0; k < partList.length; k++){
        Art* part = partList[k];
        if(part->uvCoord.length){
            M_NurbsPatch& patchOwner = part->patchOwner;
            for(int j = 0; j < patchOwner.length;){
                NurbsPatch* patch = patchOwner[j];
                int counter = 0;
                while(j < patchOwner.length && patch == patchOwner[j]){
                    counter++;
                    j++;
                }
                smlsPatchOwner += counter;
                smlsPatchOwner += patch->plidx;
            }
        }
    }
}
void Seamless::encodeAllot(){
    M_Int& allot = this->allot;
    allot.length = 0;
    int partListLen = partList.length;

    for(int k = 0; k < partListLen; k++){
        Art* part = partList[k];
        const M_ControlPoint& controlPoint = part->controlPoint;
        int controlPointLen = controlPoint.length;
        if(controlPoint.length){
            allot += k;
            int numOfGroups = 0;
            int numOfGroupsIndex = allot.length;
            allot.length++;
            for(int g = 0; g < controlPointLen; ){
                int idx = controlPoint[g].i;
                NurbsPatch* cpNode = controlPoint[g].p;
                allot += cpNode->fromPoint + idx;
                int numOfPoints = numOfConsecutivePoints(controlPoint, ++g, cpNode, ++idx, controlPointLen);
                allot += numOfPoints;
                numOfGroups++;
            }
            allot[numOfGroupsIndex] = numOfGroups;
        }
    }
}
void Seamless::extraSeamlessToString(String& str){
    encodeAllot();
    encodeStitchIndex(stitchIndex, ctrlPolyPoint);

    version = g_version;
    coordIndex.length=0;
    convertToRc(build.length);
    //convertTriToVrml(smls);
    if(coordIndex.length){
        str+="\ncoordIndex ";
        coordIndex.toString(str);
    }
    if(bnspe){
        encodePatchOwner();
        if(patchOwner.length){
            str+="\npatchOwner ";
            patchOwner.toString(str);
        }
    }
    linkToString(str);
}
void Seamless::setUpInternalEffectPtrs(){
    Effect* lastEffect = dynamic_cast<Effect*>(effect.refValue().p);
    const M_SArt& skeleton = this->skeleton;
    for(int k=0; k<skeleton.length;k++){
        skeleton[k]->setUpInternalEffectPtrs(lastEffect);
    }
}
void Seamless::setUpInternalEffectPtrs2(){ //this (2) wont be needed when setUpInternalEffectPtrs is fixed
    Effect* lastEffect=dynamic_cast<Effect*>(effect.refValue().p);
    effectList.length = 0;
    addEffect(effectList, lastEffect);

    const M_SArt& skeleton = this->skeleton;
    for(int k=0; k<skeleton.length;k++){
        skeleton[k]->setUpInternalEffectPtrs2(lastEffect);
    }
}
void Seamless::processProduct(Art* part, VertexGeometryPtr& geometry, M_Int& partTree, M_Vec3f& scalePart, M_Rotation& o, M_Vec3f& translatePart, M_Vec3f& pivotPoint, Vec3f accumScale, Matrix accumMat, Matrix accumRotMat, int matrixStack){

    if(part->show==false&&part->relevantPart.length==1)return;
    Coordinate* coordinate = dynamic_cast<Coordinate*>(geometry->coord.p);

    M_Vec3f& ifsPoint = coordinate->point;
    Matrix matrix;
    M_Vec3f& coord=part->coord.refValue();
    M_Int& scratchC=part->scratchC;

    int fromCoord=ifsPoint.length;
    int toCoord=fromCoord+part->coord.length;
    ifsPoint.length=toCoord;

    M_Int& vCoordGroup=part->vCoordGroup;


    if(part->normal.length){
        if(geometry->normal==0){
            geometry->normal=new Normal;
        }
        Normal* normalNode = dynamic_cast<Normal*>(geometry->normal.p);
        M_Vec3f& vector = normalNode->vector.refValue();
        vector.length=ifsPoint.length;
        M_Vec3f& normal=part->normal.refValue();

        if(outputNode && g_outputPovRay==false){
            for(int i=0; i<normal.length; i++){
                vector[fromCoord+i]=normal[scratchC[i]];
            }
        }
        else{
            for(int  e=0, i=0; i<vCoordGroup.length; i++){
                Rotation rot=part->rotation;
                if(i<part->weight.length){
                    rot.angle*=part->weight[i];
                }
                Matrix rotMatrix=rotationMatrix(rot)*accumRotMat;
                for(int j=0; j<vCoordGroup[i]; j++, e++){
                    vector[fromCoord+e]=rotMatrix*normal[scratchC[e]];
                }
            }
        }
    }
    accumRotMat=rotationMatrix(part->rotation)*accumRotMat;

    if(g_genColorIndex==false){
        if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(effect.p)){
            if(colorEffect->diffuse==Color3f(.8,.8,.8)){
                if(geometry->color==0){
                    geometry->color=new Color;
                }
                Color* ifsColor = dynamic_cast<Color*>(geometry->color.refValue().p);

                M_Color3f& colorPoint = ifsColor->color.refValue();
                colorPoint.length=ifsPoint.length;
                M_Color3f& color=part->color;
                for(int i=0; i<color.length; i++){
                    colorPoint[fromCoord+i]=color[scratchC[i]];
                }
            }
        }
        else if(part->workEffect->getType()==TEXTUREEFFECT){
            if(geometry->texCoord==0){
                geometry->texCoord=new TextureCoordinate;
            }
            TextureCoordinate* ifsTexCoord = dynamic_cast<TextureCoordinate*>(geometry->texCoord.p);
            M_Vec2f& texPoint = ifsTexCoord->point.refValue();
            texPoint.length=ifsPoint.length;
            M_Vec2f& texCoord=part->texCoord;
            for(int i=0; i<texCoord.length; i++){
                texPoint[fromCoord+i]=texCoord[scratchC[i]];
            }
        }
        else if(part->workEffect->getType()==MONOCOLOREFFECT){
        }
        else throw this;
    }

    if(animCP){
        for(int i=0; i<coord.length; i++){
            ifsPoint[fromCoord+i]=coord[scratchC[i]];
        }
    }
    else if(g_genHAnimNodes){
        Matrix matrix=part->matrix;
        for(int i=0; i<coord.length; i++){
            ifsPoint[fromCoord+i]=matrix*coord[scratchC[i]];
        }
    }
    else if(outputNode && g_outputPovRay==false){//bxx animation
        if(addScaleField){
            matrix=accumMat;
        }
        else{
            matrix=accumMat*scaleMatrix(part->scale);
            accumScale*=part->scale.refValue();
            accumMat=scaleMatrix(accumScale);
            if(animCP == false){
                for(int i=0; i<coord.length; i++){
                    ifsPoint[fromCoord+i]=matrix*coord[scratchC[i]];
                }
            }
        }
    }
    else{//static output
        for(int  e=0, i=0; i<vCoordGroup.length; i++){
            Rotation rot=part->rotation;
            if(i<part->weight.length){
                rot.angle*=part->weight[i];
            }
            matrix=scaleMatrix(part->scale)*rotationMatrix(rot)*translationMatrix(part->translation)*accumMat;
            for(int j=0; j<vCoordGroup[i]; j++, e++){
                ifsPoint[fromCoord+e]=matrix*coord[scratchC[e]];
            }
        }
        accumMat=scaleMatrix(part->scale)*rotationMatrix(part->rotation)*translationMatrix(part->translation)*accumMat;
    }
    matrixStack++;
    for(int  i=0; i<vCoordGroup.length; i++){
        if(addScaleField){
            scalePart+=part->scale;
            translatePart+=part->translation;
            pivotPoint+=part->pivotPoint;
        }
        else{
            translatePart+=part->translation*accumScale;
            pivotPoint+=part->pivotPoint*accumScale;;
        }
        Rotation rot=part->rotation;
        if(i<part->weight.length){
            rot.angle*=part->weight[i];
        }
        o+=rot;
        partTree+=matrixStack;
        partTree+=fromCoord;
        fromCoord+=vCoordGroup[i];
        partTree+=fromCoord;
    }

    M_SArt& partChild=part->part.refValue();
    for(int k=0; k<partChild.length; k++){
        processProduct(partChild[k], geometry, partTree, scalePart, o, translatePart, pivotPoint, accumScale, accumMat, accumRotMat, matrixStack);
    }
}
void Seamless::catMatrix(Matrix matrix){
    matrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix( translation) * matrix;
    this->matrix = matrix;
    for(int k=0; k<skeleton.length;k++){
        skeleton.refValue()[k]->catMatrix(matrix);
    }
}
void Seamless::catTranslationMatrix(){
    Matrix thisMatrix;// = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix( translation);
//	Matrix thisMatrix = translationMatrix( translation);
    for(int k=0; k<skeleton.length;k++){
        skeleton.refValue()[k]->buildSceneTranslationMatrices(thisMatrix);
    }
}
void Seamless::catScaleTranslationMatrix(){
    Matrix thisMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix( translation);
    for(int k=0; k<skeleton.length;k++){
        skeleton.refValue()[k]->buildSceneTranslationMatrices(thisMatrix);
    }
}
void Seamless::catIMatrix(Matrix imatrix){
    Vec3f translation=this->translation.refValue();
    Rotation rotation=this->rotation.refValue();
    Vec3f scale=this->scale.refValue();

    imatrix = imatrix*translationMatrix(-translation) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z));
    this->imatrix = imatrix;

    M_SArt& part = skeleton.refValue();
    for(int k=0; k<part.length; k++){
        part[k]->catIMatrix(imatrix);
    }
}
void Seamless::catMatrix(M_Rotation4d& m_accumRot){
    matrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix( translation);
	Rotation4d catRot;
    for(int k=0; k < skeleton.length;k++){
        Art* part = skeleton.refValue()[k];
        part->catMatrix(matrix, catRot, m_accumRot);
    }
}
void Seamless::catIMatrix(M_Rotation4d& m_accumRot){
    Vec3f translation = this->translation.refValue();
    Rotation rotation = this->rotation.refValue();
    Vec3f scale = this->scale.refValue();

    imatrix = translationMatrix(-translation) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z));
	Rotation4d catRot;

    M_SArt& part=skeleton.refValue();
    for(int k=0; k < part.length; k++){
        Art* prt = part[k];
        prt->catIMatrix(imatrix, catRot, m_accumRot);
    }
}
void Seamless::buildPartListNoShow(){
    partList.length=0;
    int _plidx=0;
    M_SArt& children=skeleton.refValue();
    for(int k=0; k<children.length; k++){
        children[k]->buildPartListNoShowLoop(_plidx);
    }
}
void Seamless::buildPartList(){
    partList.length=0;
    int _plidx=0;
    M_SArt& children=skeleton.refValue();
    for(int k=0; k<children.length; k++){
        children[k]->buildPartListLoop(_plidx);
    }
}
void Seamless::processLoop(Art* part, int &coordLenTally, int &_plidx, Matrix parentMatrix ){

    Matrix thisMatrix = scaleMatrix(part->scale) * rotationMatrix(part->rotation) * translationMatrix( part->translation) * parentMatrix;
    part->matrix = thisMatrix;

    part->smls = this;
    part->plidx = _plidx++;
    part->fromCoord=coordLenTally;
    coordLenTally+=part->coord.length;
    part->toCoord=coordLenTally;

    for(int j=part->morpher.length; --j>=0;){
        Morpher* morpher = const_cast<Morpher*>(part->morpher[j]);
        morpher->parent = part;
        morpher->smls = this;
    }
    part->geometry = geometry;

    partList += part;

    M_SArt& m_part = part->part.refValue();

    for(int k=0; k < m_part.length; k++){
        Art* child = m_part[k];
        child->parent = part;
        processLoop(child, coordLenTally, _plidx, thisMatrix);
    }
}
void Seamless::process(){

    if(!skeleton.length){
        partList.length=0;
        return;
    }
    partList.length = 0;
    int coordLenTally = 0;
    int plidx = 0;

    Matrix thisMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix( translation);
    for(int k=0; k<skeleton.length; k++){
        Art* part = skeleton.refValue()[k];
//		part->parent=this;
        processLoop(part, coordLenTally, plidx, thisMatrix );
        part->calcAbsolutePivotPoint(Vec3f());
    }
    //inState=false;
    //if(autoIn){
    //	setAllIns(build.refValue(), false);
    //}

    this->catIMatrix();
    setUpInternalEffectPtrs();
    if(actTriPart!=0){
        if(actTriPart->plidx<partList.length){
            if(actTriPart==partList[actTriPart->plidx]){
                if(ActTri>=actTriPart->part.length){
                    ActTri=0;
                }
            }
            else{
                actTriPart=0;
            }
        }
        else{
            actTriPart=0;
        }
    }
    if(xRay){
        if(stickMan == 0){
            stickMan = new StickMan();
        }
        stickMan->process(this);
    }
    ctrf=true;

//	orderPolygonsFromVertices();
//	convertTo028();
//	convertTo027();
}
void Seamless::fillLegalCoordGroups(){
    for(int k = partList.length; --k>=0;){
        Art* part = partList[k];
        M_Vec3f& coord = part->coord.refValue();
        M_Int& coordGroup = part->coordGroup.refValue();
        M_Int& legalCoordGroup = part->legalCoordGroup;
        int c = 0;
        int coordGroupLen = coordGroup.length;
        for(int i = 0; i < coordGroupLen; i++){
            int b = coordGroup[i];
            if(c + b > coord.length){
                b = coord.length - c;
                if(b <= 0){
                    break;
                }
                legalCoordGroup+=part->coord.length - c;
                break;
            }
            legalCoordGroup+=b;
            c+=b;
        }
    }
}
bool Seamless::checkUsingPivotPoints(){
    bool state = false;
    M_SArt& skeleton = this->skeleton.refValue();
    for(int k = 0; k < skeleton.length && state == false; k++){
        skeleton[k]->checkUsingPivotPoints(state);
    }
    return state;
}
void Seamless::convertPivotPointsToTranslationsNoChangePivotPoint(){
    M_SArt& skeleton = this->skeleton.refValue();
    for(int k = 0; k < skeleton.length; k++){
        skeleton[k]->convertPivotPointsToTranslationsNoChangePivotPoint(Vec3f());
    }
}
void Seamless::resetFromCoordAndWorkoutVerticesForGroupCoords(){
    int linCoordLen=0;
    for(int k=0; k<partList.length; k++){
        Art* part=partList[k];
        part->fromCoord=linCoordLen;
        linCoordLen+=part->coord.length;
        if(part->getType()==PART||part->getType()==IMMEDIATEPART){
            part->workoutVerticesForGroupCoords();
        }
    }
}
void Seamless::convertToColorIndex(IndexedFaceSet* ifs){
    defragEffect();
    {
        for(int k=0; k<partList.length; k++){
            Art* part=partList[k];
            part->vCoordGroup.length=0;
        }
    }

    resetFromCoordAndWorkoutVerticesForGroupCoords();

    switch(effect->getType()){
    case COLOREFFECT:
    {
        ifs->refColor()=dynamic_cast<ColorEffect*>(effect.p)->color;
        M_Int& colorIndex=ifs->colorIndex.refValue();
        colorIndex.length=ifs->coordIndex.getValue().length;
        for(int j=0, k=0; k<partList.length; k++){
            Art* part=partList[k];
            M_Triangle& triangle=part->triangle;
            int triLen=triangle.length*3;
            for(int i=0; i<triLen; ){
                colorIndex[j++]=triangle.refTexIndex(i++);
                colorIndex[j++]=triangle.refTexIndex(i++);
                colorIndex[j++]=triangle.refTexIndex(i++);
                colorIndex[j++]=-1;
            }
        }
    }
    break;
    case TEXTUREEFFECT:
    {
        ifs->refTexPoint()=dynamic_cast<TextureEffect*>(effect.p)->texCoord;
        M_Int& texCoordIndex=ifs->texCoordIndex.refValue();
        texCoordIndex.length=ifs->coordIndex.getValue().length;
        for(int j=0, k=0; k<partList.length; k++){
            Art* part=partList[k];
            M_Triangle& triangle=part->triangle;
            int triLen=triangle.length*3;
            for(int i=0; i<triLen; ){
                texCoordIndex[j++]=triangle.refTexIndex(i++);
                texCoordIndex[j++]=triangle.refTexIndex(i++);
                texCoordIndex[j++]=triangle.refTexIndex(i++);
                texCoordIndex[j++]=-1;			}
        }
    }
    break;
    default:
        throw effect;
    }
}
void Seamless::outputToString(String& str, bool allowRef){
    fillLegalCoordGroups();
    TransformPtr transform=new Transform;

    bool usingPivotPoints = checkUsingPivotPoints();

    if(animCP == false){
        if(g_genHAnimNodes==false && usingPivotPoints){
            convertPivotPointsToTranslationsNoChangePivotPoint();
        }
        else if(g_genHAnimNodes && usingPivotPoints){
            for(int k = partList.length; --k>=0;){
                partList[k]->translation.refValue() = Vec3f();
                //partList[k]->rotation.refValue() = Rotation();
            }
            //process();
        }
    }


    bool transformExist=false;
    if(animCP == false){
        if(g_outputPovRay == false && outputNode && ( name!=""||scale.refValue()!=Vec3f(1,1,1)||rotation.refValue().angle!=0||translation!=Vec3f())){
            transformExist=true;
            if(allowRef){
                transform->name=name;
            }
            transform->scale.refValue()=scale.refValue();
            transform->rotation.refValue()=rotation;
            transform->translation.refValue()=translation;
        }
    }
    M_Node& m_shape=transform->children;
    m_shape.length=0;

    const M_SArt& skeleton=this->skeleton;
    if(skeleton.length==0){
        return;
    }
    if(animCP == false){
        this->catTranslationMatrix();
    }

    Shape* shape=new Shape;
    VertexGeometryPtr geometry;
    if(g_genIdxTriSetNodes){
        geometry=new IndexedTriangleSet;
    }
    else{
        geometry=new IndexedFaceSet;
    }

    if(partList.length==0)return;
    Effect* effect=dynamic_cast<Effect*>(this->effect.getRawPtr());

    effect->copyFieldsData(shape);

    Color* color=0;
    if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(effect)){
        if(colorEffect->diffuse==Color3f(.8,.8,.8)){
            color=new Color;
            geometry->color=color;
            geometry->solid = !colorEffect->backfaceRender;
            color->name='t';
            makeNodeNameFromAddr(color->name, this);
        }
    }
    else if(TextureEffect* textureEffect = dynamic_cast<TextureEffect*>(effect)){
        TextureCoordinate* texCoord=new TextureCoordinate;
        geometry->texCoord=texCoord;
        geometry->solid = !textureEffect->backfaceRender;
        texCoord->name='t';
        makeNodeNameFromAddr(texCoord->name, this);
    }
    else if(effect->getType()==MONOCOLOREFFECT){
    }
    else {
        throw effect;
    }

    CoordinatePtr coord=new Coordinate;
    coord->name='c';
    makeNodeNameFromAddr(coord->name,this);
    geometry->coord=coord;

    NormalPtr normal=new Normal;
    normal->name='n';
    makeNodeNameFromAddr(normal->name,this);
    geometry->normal=normal;

//	geometry->creaseAngle=3.14;
    shape->geometry=geometry;


    M_Vec3f scalePart;
    M_Vec3f translatePart;
    M_Vec3f pivotPoint;
    M_Int partTree;

    geometry->genTransformedNormals(partList);

    IndexedFaceSet* ifs = dynamic_cast<IndexedFaceSet*>(geometry.p);
    if(g_genColorIndex && ifs){
        convertToColorIndex(ifs);
    }
    else{
        geometry->addExtraCoords(this);
        if(effect->getType()==COLOREFFECT && geometry->color.p){
            dynamic_cast<Color*>(geometry->color.p)->color.refValue().length=0;
        }
        if(effect->getType()==TEXTUREEFFECT){
            dynamic_cast<TextureCoordinate*>(geometry->texCoord.p)->point.refValue().length=0;
        }
    }
    if(dynamic_cast<MonoColorEffect*>(effect)==0){
        dynamic_cast<Coordinate*>(geometry->coord.p)->point.refValue().length=0;
        dynamic_cast<Normal*>(geometry->normal.p)->vector.refValue().length=0;
    }

    MFRotation* o = new MFRotation(M_Rotation());


    Vec3f scale2=Vec3f(1,1,1);
    Matrix matrix;
    if(transformExist==false){
        scale2=scale.refValue();
    }
    if(outSegmented==false){
        if(outputNode==0 || g_outputPovRay){
            matrix=scaleMatrix(scale2)*rotationMatrix(rotation)*translationMatrix(translation);
        }
        for(int k=0; k<skeleton.length; k++){
            processProduct(skeleton[k], geometry, partTree, scalePart, o->refValue(), translatePart, pivotPoint, scale2, matrix, rotationMatrix(rotation));
        }
        if(g_genNormals==false){
            geometry->normal=0;
            IndexedFaceSet* ifs=dynamic_cast<IndexedFaceSet*>(geometry.p);
            if(ifs!=0){
                ifs->creaseAngle=3.14;
            }
        }
        //group effect triangles
        {
            for(int k=this->partList.length; --k>=0;){
                Effect* effect=partList[k]->effect;
                if(effect!=nullPtr){
                    effect->outputShape=nullPtr;
                }
            }
        }
        m_shape+=shape;
        {
            effect->outputShape=shape;
            for(int k=0; k<skeleton.length; k++){
                int tmp=0;
                geometry->groupTrianglesByEffect(skeleton[k], m_shape, tmp, effect);
            }
        }
        if(m_shape.length>1){
            geometry->deleteUnwantedTriFromSource();
        }
        else{
            if(geometry->color!=nullPtr){
                geometry->color->name="";
            }
            if(geometry->texCoord!=nullPtr){
                geometry->texCoord->name="";
            }
        }
    }
    if(outSegmented==false && g_genHAnimNodes==false){
        if(allowRef){
            if(transformExist){
                transform->name = name;
            }
            else if(m_shape.length>1){
                GroupPtr group = new Group;
                group->name = name;
                group->children = m_shape;
                m_shape = M_Node(group.p);
            }
            else{
                shape->name = name;
            }
        }
        if(g_writeX3DXML){
            if(transformExist){
                transform->toXMLString(str, false, "");
            }
            else{
                for(int k=0; k<m_shape.length; k++){
                    m_shape[k]->toXMLString(str, false, "");
                }
            }
        }
        else {
            if(transformExist){
                transform->toString(str, false);
            }
            else {
                for(int k=0; k<m_shape.length; k++){
                    //if(g_writeObj){
                    //	dynamic_cast<Shape*>(m_shape[k].p)->toObjString(str);
                    //}
                    if(g_outputPovRay){
                        dynamic_cast<Shape*>(m_shape[k].p)->toPOVRayString(str);
                    }
                    else{
                        m_shape[k]->toString(str,false);
                    }
                }
            }
        }
    }
    if(Transform* transform=dynamic_cast<Transform*>(outputNode.p)){
        if(allowRef){
            transform->name = name;
        }
        if(g_writeX3DXML){
            transform->toXMLString(str, false, 0);//continer field not right
        }
        else {
            transform->toString(str,false);
        }
    }
    else if(HAnimHumanoid* humanoid=dynamic_cast<HAnimHumanoid*>(outputNode.p)){
        dynamic_cast<Node*>(humanoid)->name = name;
        humanoid->skin.refValue()+=m_shape;
        humanoid->skinCoord=geometry->coord;
        humanoid->skinNormal=geometry->normal;
        int skinCoordIndexCntr=0;

        M_Node& hskeleton=humanoid->skeleton.refValue();
        for(int k=0, jk=0; k<skeleton.length;k++){
            Art* part=skeleton[k];
            if(part->show==false&&part->relevantPart.length==1)continue;
            HAnimJoint* joint=dynamic_cast<HAnimJoint*>(hskeleton[jk].p);
            if(usingPivotPoints){
                joint->center=part->absolutePivotPoint;
            }
            else{
                joint->center=part->matrix*Vec3f();
            }

            M_Int& skinCoordIndex=joint->skinCoordIndex.refValue();
            M_Float& skinCoordWeight=joint->skinCoordWeight.refValue();
            skinCoordWeight.length=part->coord.length;
            skinCoordIndex.length=part->coord.length;
            for(int i=0; i<skinCoordWeight.length; i++ ){
                skinCoordWeight[i]=1;
                skinCoordIndex[i]=skinCoordIndexCntr;
                skinCoordIndexCntr++;
            }
            {
                const M_Morpher& m_morpher=part->morpher;
                for(int i=0; i<m_morpher.length; i++){
                    const Morpher* morpher=m_morpher[i];
                    HAnimDisplacer* hAnimDisplacer=new HAnimDisplacer;
                    hAnimDisplacer->name=dynamic_cast<Node*>(hAnimDisplacer)->name=morpher->name;
                    hAnimDisplacer->displacements.refValue()=morpher->displacement;
                    hAnimDisplacer->coordIndex.refValue()=joint->skinCoordIndex;
                    joint->displacers.refValue()+=hAnimDisplacer;
                }
            }
            genHAnimSkinCoordWeight(part, joint, skinCoordIndexCntr, usingPivotPoints);
            jk++;
        }
        if(g_writeX3DXML){
            humanoid->toXMLString(str, false,"children");//continer field not right
        }
        else {
            humanoid->toString(str,false);
            //if(color){
            //	outO3D(str, geometry, color);
            //}
        }
    }
    else if(outputNode){
        changeFloatFormat(6);
        Script* script=dynamic_cast<Script*>(outputNode.getRawPtr());
        script->url.refValue()[0]+=addTransformScript(this, partTree);
        M_Properties& properties=script->properties;

/*
        MFRotation* o = new MFRotation(M_Rotation());
        M_Rotation& _o=o->refValue();
        _o.length=translatePart.length;
        for(int k=0, j=0; k<partList.length; k++){
            Art* part=partList[k];
            if(part->getType()!=VERTEXBANK){
                int coordCnt=0;
                const M_Float& weight=part->weight;
                const M_Int& vCoordGroup=part->vCoordGroup;
                for(int h=0; h<vCoordGroup.length; h++){
                    coordCnt+=vCoordGroup[h];
                    Rotation rotation=part->rotation;
                    rotation.angle*=weight[h];
                    _o[j++]=rotation;
                }
                if(coordCnt<part->coord.length){
                    _o[j++]=part->rotation;
                }
            }
        }
*/
        dynamic_cast<IndexedFaceSet*>(shape->geometry.p)->color=0;
        if(script){
            {
                for(int k=partList.length; --k>=0;){
                    const Art* part=partList[k];
                    const M_Morpher& morpher=part->morpher;
                    for(int k=morpher.length; --k>=0;){
                        addMorpherToScript(script, properties, part, morpher[k], script->url.refValue()[0]);
                    }
                }
            }

            if(g_writeX3DXML||g_writeX3DVRMLClassic||g_genContactSBVT) //exposed fields for script if legal or for contact
            {
                *script+=new MFInt32(M_Int(partTree));
                *script+=new MFVec3f(M_Vec3f(translatePart));
                *script+=new MFVec3f(M_Vec3f(pivotPoint));
                *script+=o;
                if(addScaleField){
                    *script+=new MFVec3f(M_Vec3f(scalePart));
                }
            }
            else
            {
                *script+=new MFInt32(M_Int(partTree));
                *script+=new MFVec3f(M_Vec3f(translatePart));
                *script+=new MFVec3f(M_Vec3f(pivotPoint));
                *script+=o;
            }

            properties+=Properties("partTree");
            properties+=Properties("translatePart");
            properties+=Properties("pivotPoint");
            properties+=Properties("o");
            if(addScaleField){
                properties+=Properties("scalePart");
            }

            *script+=new MFVec3f(M_Vec3f());
            properties+=Properties("blankTranslate");

///////////////

            *script+=new SFNode( geometry->coord.refValue());
            properties+=Properties("crd");

            if(g_genNormals){
                *script+=new SFNode( geometry->normal.refValue());
                properties+=Properties("nrm");
            }

            *script+=new MFVec3f(M_Vec3f());
            properties+=Properties("scrd");

            *script+=new MFVec3f( M_Vec3f());
            properties+=Properties("snrm");

            *script+=new MFVec3f( M_Vec3f());
            properties+=Properties("dcrd");

            *script+=new MFVec3f( M_Vec3f());
            properties+=Properties("dnrm");

            if(g_writeX3DXML){
                script->toXMLString(str, false, "children");
            }
            else {
                script->toString(str,false);
            }
        }
    }
    if(g_genColorIndex==false){
        resetCoordLen(); //fix up what addExtraCoords changed
    }
    this->catMatrix();
}
void Seamless::orderPolygonsFromVertices(){
    if(geometry==NULL)return;
    int poly=0;
    int search;
    Art* part;
    int polyLength=IndexedFaceSetPtr(geometry)->coordIndex.length/4;

    for(int k=0;k<partList.length;k++){
        part=partList[k];
        part->fromPoly=poly;
        while(1){
            if( poly < polyLength && part->polyBelongs( poly ) ) {
                poly++;
                continue;
            }
            else {
                if(part->findNextPolyThatBelongs(search=poly+1)){
                    part->geometry->swapTriangles(poly, search);
                    poly++;
                }
                else {
                    part->toPoly=poly;
                    break;
                }
            }
        }
    }
}
void Seamless::defragEffect(){
    if(partList.length==0)return;
    Effect* effect=partList[0]->workEffect;
    M_Int& lookup=effect->lookup;
    lookup.length=0;
    int effectLen=effect->getEffectLength();
    if(effectLen){
        lookup.length=effectLen;

        for(int k=partList.length;--k>=0;){
            M_Triangle& triangle=partList[k]->triangle;
            for(int i=triangle.length*3;--i>=0;){
                lookup[triangle.refTexIndex(i)]=1;
            }
        }
        if(effect->getType()==COLOREFFECT){
            M_Color3f& color=dynamic_cast<ColorEffect*>(effect)->color.refValue();
            int c=0;
            for(int i=0; i<effectLen;i++){
                int lup=lookup[i];
                lookup[i]=c;
                if(lup){
                    color[c]=color[i];
                    c+=lup;
                }

            }
            color.length=c+1;
        }
        else if(effect->getType()==TEXTUREEFFECT){
            M_Vec2f& texCoord=dynamic_cast<TextureEffect*>(effect)->texCoord.refValue();
            int c=0;
            for(int i=0; i<effectLen;i++){
                int lup=lookup[i];
                lookup[i]=c;
                if(lup){
                    texCoord[c]=texCoord[i];
                    c+=lup;
                }

            }
            texCoord.length=c+1;
        }
        else throw this;
        {
            for(int k=partList.length;--k>=0;){
                M_Triangle& triangle=partList[k]->triangle;
                for(int i=triangle.length*3;--i>=0;){
                    triangle.refTexIndex(i)=lookup[triangle.refTexIndex(i)];
                }
            }
        }
        lookup.length=0;
    }
}
void Seamless::createContigLookUpList(M_<NurbsPatch*>& contigLookUpList){
    for(int k = 0; k < cpNodeList.length; k++){
        NurbsPatch* patch = cpNodeList[k];
        patch->fromPoint = contigLookUpList.length;
        for(int i = patch->point.length; --i >= 0;){
            contigLookUpList += patch;
        }
    }
    pointTotal = contigLookUpList.length;
}
void Seamless::cpNodeListAddForStem(M_Node& m_stem){
    for(int k = 0; k < m_stem.length; k++){
        if(Stem* stem = dynamic_cast<Stem*>(m_stem[k].p)){
            stem->mirrorStem = 0;
            M_Node& m_fuse = stem->fuse.refValue();
            for(int k = 0; k < m_fuse.length; k++){
                if(FuseSurface* fuse = dynamic_cast<FuseSurface*>(m_fuse[k].p)){
                    if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(fuse->patch.p)){
                        cpNodeList += patch;
                        cpNodeListAddForStem(patch->stem.refValue());
                    }
                }
            }
        }
    }
}
void Seamless::cpNodeListAdd(){
    nurbsLine.length = 0;
    cpNodeList.length = 0;
    pointTotal = 0;
    for(int k = 0; k < build.length; k++){
        build.refValue()[k]->cpNodeListAdd(this);
    }
    setPointFrom();
}
void Seamless::resetCoordLen(){
    for(int k = 0; k < partList.length; k++){
        Art* part=partList[k];
        part->coord.refValue().length=part->toCoord;
        part->normal.refValue().length=part->toCoord;
        part->color.length=0;
        part->texCoord.length=0;
    }
}
void Seamless::setPointFrom(){
    pointTotal = 0;
    for(int k = 0; k < cpNodeList.length; k++){
        NurbsPatch* patch = cpNodeList[k];
        patch->plidx = k;
        patch->fromPoint = pointTotal;
        pointTotal += patch->point.length;
        patch->smls = this;
        patch->mirrorPatch = 0;
        patch->isCtrlPolyRef = false;
    }
}
void Seamless::flagCtrlPolyPointRefs(){
    for(int k = ctrlPolyPoint.length; --k >= 0;){
        if(NurbsPatch* patch = ctrlPolyPoint[k].p){
            patch->isCtrlPolyRef = true;
        }
    }
}
void Seamless::decodeLink(M_<NurbsPatch*>& contigLookUpList){
    for(int k = link.length; --k>=0;){
        Link& lnk = link[k];
        lnk.patch = contigLookUpList[lnk.point];
        lnk.point -= lnk.patch->fromPoint;
    }
}
void Seamless::decodeAllot(){
//	if(allot.length == 0){
//		return;
//	}
	M_<NurbsPatch*> contigLookUpList;
    createContigLookUpList(contigLookUpList);

    for(int o = 0; o < allot.length;){
        Art* part = partList[allot[o++]];
        M_ControlPoint& controlPoint = part->controlPoint;

        int numOfGroups = allot[o++];

        while(numOfGroups--){
            int idx = allot[o++];
            int numOfPoints = allot[o++];

            while(numOfPoints--){
                NurbsPatch* patch = contigLookUpList[idx];
                controlPoint += ControlPoint(patch, idx - patch->fromPoint);
                idx++;
            }
        }
    }
    decodeStitchIndex(contigLookUpList);
    decodeLink(contigLookUpList);
}
void Seamless::updateCage(){
    if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
        if(part->smls == this){
            g_partCageClue->genCage();
        }
    }
}

void Seamless::addCageControlPolys(){
	M_Vec3f& point=dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();

	int ctrlPolyPointLen = ctrlPolyPoint.length;
	for(int i = 0; i < ctrlPolyPointLen;){
		int ctrlPoly = i;
		int len = ctrlPolyPoint[ctrlPoly].len;
		int mark = coordIndex.length;
		bool flag = false;
		int m = i + len;
		for(int k = len; --k >= 0;){
			NurbsPatch* patch = ctrlPolyPoint[i].p;
			flag |= patch->cage;
			coordIndex += patch->fromPoint + ctrlPolyPoint[i].i;
			i++;
		}
		if(flag == false){
			coordIndex.length = mark;
		}
		else{
			coordIndex += ctrlPolyPoint[ctrlPoly].p->fromPoint + ctrlPolyPoint[ctrlPoly].i;
			coordIndex += -1;
		}
		i = getNextCtrlPoly(ctrlPoly);
	}
	if(g_scene->mode & STITCH_POLY_MODE && ctrlPolyPoint.length){
		genTransparentCtrlPolys(g_vcNurbsCtrlPolyPart, point);
	}
}

inline bool atLeastOnePatchIsVisible(const M_CtrlPolyPoint& ctrlPolyPoint, int i, const int len){
	for(int k = len; --k >= 0;){
		if(ctrlPolyPoint[i++].p->cage){
			return true;
		}
	}
	return false;
}
/*
bool Seamless::isCrossOver(int midIdx){
	int len = ctrlPolyPoint[midIdx].len;
	if(len != 2){
		return false;
	}
	if(ctrlPolyPoint[midIdx].p == ctrlPolyPoint[midIdx + 1].p && ctrlPolyPoint[midIdx].i == ctrlPolyPoint[midIdx + 1].i){
		return false;
	}
	return true;
}
*/
bool Seamless::isCrossOver(int midIdx, const NurbsPatch* patchA, const int actIdxA, const NurbsPatch* patchB, int& midActIdxA, int& midActIdxB)const{
	int len = ctrlPolyPoint[midIdx].len;
	if(len >= 2){
		for(int j = len; --j >= 0;){
			if(ctrlPolyPoint[midIdx + j].p == patchA){
				midActIdxA = ctrlPolyPoint[midIdx + j].i;
				for(int k = len; --k >= 0;){
					if(j != k){
						if(ctrlPolyPoint[midIdx + k].p == patchB){
							midActIdxB = ctrlPolyPoint[midIdx + k].i;
							if(patchA == patchB){
								if(midActIdxA == midActIdxB){
									continue;
								}
								int diffA = abs(midActIdxA - actIdxA);
								int diffB = abs(midActIdxB - actIdxA);
								if(diffB < diffA){
									swap(midActIdxA, midActIdxB);
								}
							}
							return true;
						}
					}
				}
			}
		}
	}
	return false;
}

void Seamless::addCageControlPolysColorCode(){
	M_Vec3f& point=dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
	M_Color3f& color = dynamic_cast<Color*>(g_nurbsIlsColor)->color;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();

	int ctrlPolyPointLen = ctrlPolyPoint.length;
	int lookUpIndex = 0;
	int actIdxA = 0;
	int visIdxA = 0;
	NurbsPatch* patchA = 0;

	int actIdxB = 0;
	int visIdxB = 0;
	NurbsPatch* patchB = 0;

	for(int i = 0; i < ctrlPolyPointLen;){
		int ctrlPoly = i;
		int len = ctrlPolyPoint[ctrlPoly].len;
		int midPoly = ctrlPoly + len;
		int m = midPoly;
		int midLen = ctrlPolyPoint[m].len;


		int mark = coordIndex.length;
		if(false == atLeastOnePatchIsVisible(ctrlPolyPoint, i, len)){
			i = getNextCtrlPoly(ctrlPoly);
			continue;
		}

		NurbsPatch* patchA = ctrlPolyPoint[ctrlPoly].p;
		actIdxA = ctrlPolyPoint[ctrlPoly].i;
		visIdxA = patchA->convertActualToVisual(actIdxA, patchA->dim.x, patchA->dim.y);


		i++;

		for(int k = 1; k < len + 1;){
			if(k == len){
				patchB = ctrlPolyPoint[ctrlPoly].p;
				actIdxB = ctrlPolyPoint[ctrlPoly].i;
			}
			else{
				patchB = ctrlPolyPoint[i].p;
				actIdxB = ctrlPolyPoint[i].i;
			}
			visIdxB = patchB->convertActualToVisual(actIdxB, patchB->dim.x, patchB->dim.y);
			k++;

			int uDimA = patchA->dim.x;
			int vDimA = patchA->dim.y;

			int uDimB = patchB->dim.x;
			int vDimB = patchB->dim.y;

			const int uCellsA = (uDimA / 2);
			const int vCellsA = (vDimA / 2);

			const int uCellsB = (uDimB / 2);
			const int vCellsB = (vDimB / 2);

			int vVisA = visIdxA / (uCellsA + 2);
			int uVisA = visIdxA - vVisA * (uCellsA + 2);

			int vVisB = visIdxB / (uCellsB + 2);
			int uVisB = visIdxB - vVisB * (uCellsB + 2);

			int midActIdxA = 0;
			int midActIdxB = 0;

			if(isCrossOver(m, patchA, actIdxA, patchB, midActIdxA, midActIdxB)){

				int midVisIdxA = patchA->convertActualToVisual(midActIdxA, patchA->dim.x, patchA->dim.y);
				int midVisIdxB = patchB->convertActualToVisual(midActIdxB, patchB->dim.x, patchB->dim.y);

				int vMidVisA = midVisIdxA / (uCellsA + 2);
				int uMidVisA = midVisIdxA - vMidVisA * (uCellsA + 2);

				int vMidVisB = midVisIdxB / (uCellsB + 2);
				int uMidVisB = midVisIdxB - vMidVisB * (uCellsB + 2);

				if(vVisA == vMidVisA){
					if(uVisA > uMidVisA){
						swap(uVisA, uMidVisA);
					}
					patchA->uAddControlPolyLine(uVisA, vVisA, g_nurbsIls);
				}
				else if(uVisA == uMidVisA){
					if(vVisA > vMidVisA){
						swap(vVisA, vMidVisA);
					}
					patchA->vAddControlPolyLine(uVisA, vVisA, g_nurbsIls);
				}
				if(vVisB == vMidVisB){
					if(uVisB > uMidVisB){
						swap(uVisB, uMidVisB);
					}
					patchB->uAddControlPolyLine(uVisB, vVisB, g_nurbsIls);
				}
				else if(uVisB == uMidVisB){
					if(vVisB > vMidVisB){
						swap(vVisB, vMidVisB);
					}
					patchB->vAddControlPolyLine(uVisB, vVisB, g_nurbsIls);
				}
			}
			else if(patchA == patchB){
				if(vVisA == vVisB){
					if(uVisA > uVisB){
						swap(uVisA, uVisB);
					}
					patchA->uAddControlPolyLine(uVisA, vVisA, g_nurbsIls);

					//lookUpIndex = g_pointLookup[patchA->plidx];
					//coordIndex += lookUpIndex + visIdxA;

					//lookUpIndex = g_pointLookup[patchB->plidx];
					//coordIndex += lookUpIndex + visIdxB;

					//coordIndex += -1;
				}
				else if(uVisA == uVisB){
					if(vVisA > vVisB){
						swap(vVisA, vVisB);
					}
					patchA->vAddControlPolyLine(uVisA, vVisA, g_nurbsIls);
				}
			}

			if(midLen == 0){
				m++;
			}
			else{
				m += midLen;
			}
			midLen = ctrlPolyPoint[m].len;

			i++;

			patchA = patchB;
			actIdxA = actIdxB;
			visIdxA = visIdxB;

		}
		if(midLen == 0){//midLen is centreLen
			m++;
		}
		else{
			m += midLen;
		}
		i = m;
	}

	//if(g_scene->mode & STITCH_POLY_MODE && ctrlPolyPoint.length){
	//	genTransparentCtrlPolys(g_vcNurbsCtrlPolyPart, point);
	//}
}
int Seamless::getCtrlPolyLengthInTri(int& coordLen){
	int triLen = 0;
	for(int i = 0; i < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[i].len;
		coordLen += len;
		triLen += len - 2;
		i = getNextCtrlPoly(i);
	}
	return triLen;
}
void Seamless::genTransparentCtrlPolys(Art* part, const M_Vec3f& point){
	if(part->workEffect==0)return;
	part->deleteGeometry();
	Seamless* smls = part->smls;
	Effect* effect=part->workEffect;
	int effectLength=effect->getEffectLength();
	int coordLen = 0;
	int triLen = getCtrlPolyLengthInTri(coordLen);
	M_Color3f& color = dynamic_cast<ColorEffect*>(effect)->color.refValue();
	color.length = coordLen;
	for(int i = coordLen; --i >= 0;){
		color[i] = Color3f(0,1,1);
	}

	M_Vec3f& coord = part->coord.refValue();
	coord.length = coordLen;
	M_Triangle& m_triangle = part->triangle;
	m_triangle.length = triLen;

	int ctrlPolyPointLen = ctrlPolyPoint.length;

	for(int i = 0, t = 0, c = 0, s = 0; i < ctrlPolyPointLen;){
		int len = ctrlPolyPoint[i].len;
		if(s < MultiSelect::ctrlPolySelList.length){
			if(i == MultiSelect::ctrlPolySelList[s]){
				for(int j = 0; j < len; j++){
					color[c + j] = Color3f(1,0,0);
				}
				s++;
			}
		}
		int c0 = c;
		coord[c0] = point[ctrlPolyPoint[i + 0].p->fromPoint + ctrlPolyPoint[i + 0].i];
		coord[c + 1] = point[ctrlPolyPoint[i + 1].p->fromPoint + ctrlPolyPoint[i + 1].i];
		coord[c + 2] = point[ctrlPolyPoint[i + 2].p->fromPoint + ctrlPolyPoint[i + 2].i];
		m_triangle[t++] = S_Triangle(part, c + 0, c + 0, part, c + 1, c + 1, part, c + 2, c + 2);

		for(int j = 2; j < len - 1; j++){
			coord[c + j + 1] = point[ctrlPolyPoint[i + j + 1].p->fromPoint + ctrlPolyPoint[i + j + 1].i];
			m_triangle[t++] = S_Triangle(part, c0, c0, part, c + j, c + j, part, c + j + 1, c + j + 1);
		}
		c += len;
		i = getNextCtrlPoly(i);
	}

	part->relevantPart=M_Art(part);
	smls->process();
}
void Seamless::colorLinkPoints(){
	M_Color3f& color = dynamic_cast<Color*>(g_nurbsIlsColor)->color;

	for(int i = link.length; --i>=0;){
		color[link[i].patch->fromPoint + link[i].point] = Color3f(0,0,1);
	}
}
void Seamless::genSelectedBoxPolys(Art* part, const M_Vec3f& point){
	if(part->workEffect==0)return;
	part->deleteGeometry();
	Seamless* smls = part->smls;
	Effect* effect=part->workEffect;
	int effectLength=effect->getEffectLength();
	int coordLen = MultiSelect::boxPolySelList.length;
	int triLen = coordLen >> 1;
	M_Color3f& color = dynamic_cast<ColorEffect*>(effect)->color.refValue();
	color.length = coordLen;
	for(int i = coordLen; --i >= 0;){
		color[i] = Color3f(0,1,1);
	}

	M_Vec3f& coord = part->coord.refValue();
	coord.length = coordLen;
	M_Triangle& m_triangle = part->triangle;
	m_triangle.length = triLen;

	int boxPolySelListLen = MultiSelect::boxPolySelList.length;

	for(int i = 0, t = 0; i < coordLen; i +=4){
		coord[i + 0] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 0].i];
		coord[i + 1] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 1].i];
		coord[i + 2] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 2].i];

		m_triangle[t++] = S_Triangle(part, i + 0, i + 0, part, i + 1, i + 1, part, i + 2, i + 2);
		
		coord[i + 2] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 2].i];
		coord[i + 3] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 3].i];
		coord[i + 0] = point[MultiSelect::boxPolySelList[i].p->fromPoint + MultiSelect::boxPolySelList[i + 0].i];

		m_triangle[t++] = S_Triangle(part, i + 2, i + 2, part, i + 3, i + 3, part, i + 0, i + 0);
	}

	part->relevantPart = M_Art(part);
	smls->process();
}

void Seamless::genVisibleMask(){
	g_viewMatrix = g_scene->getViewMatrix();

	g_screenPoint.length = pointTotal;

	if(g_scene->mode & BOX_PATCH_MODELLING_MODE){
		g_pointIsVisible.length = 0;//set all to false first
		g_pointIsVisible.length = pointTotal;
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			patch->setVisibleMaskForStitchEditMode(g_pointIsVisible);
		}
	}
	else if(g_scene->mode & BACKFACE_IGNORE_MODE){
		g_pointIsVisible.length = 0;//set all to false first
		g_pointIsVisible.length = pointTotal;
		if(g_transformCP->selectObject){
			for(int k = 0; k < cpNodeList.length; k++){
				NurbsPatch* patch = cpNodeList[k];
				patch->calcScreenPoints(g_screenPoint, g_viewMatrix);
				patch->setBackfaceMaskForInnerPoints(g_pointIsVisible, g_screenPoint);
			}
		}
		else{
			for(int k = 0; k < cpNodeList.length; k++){
				Build* bld = cpNodeList[k]->container;
				if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(bld)){
					patch->calcScreenPoints(g_screenPoint, g_viewMatrix);
					patch->setBackfaceMaskForInnerPoints(g_pointIsVisible, g_screenPoint);
				}
			}
		}
		if(g_showPatchBorderMode == false){
			calcPolyCenterScreenPoints(ctrlPolyPoint, g_screenPoint, g_pointIsVisible, g_viewMatrix);
		}
	}
	else{
		if(g_transformCP->selectObject){
			for(int k = 0; k < cpNodeList.length; k++){
				cpNodeList[k]->calcScreenPoints(g_screenPoint, g_viewMatrix);
			}
		}
		else{
			for(int k = 0; k < cpNodeList.length; k++){
				Build* bld = cpNodeList[k]->container;
				if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(bld)){
					patch->calcScreenPoints(g_screenPoint, g_viewMatrix);
				}
			}
		}
	}
}
void Seamless::setModellingPatchMode(){
	if(workAnimCP){
		workAnimCP = false;
		M_<NurbsPatch*>& cpNodeList = this->cpNodeList;
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			if(patch->point.length == patch->tpoint.length){
				M_Vec3f& tpoint = patch->tpoint;
				patch->point = tpoint;
				M_Int& cpIndex = patch->cpIndex.refValue();

				for(int i = cpIndex.length; --i>=0;){
					cp.refValue()[cpIndex[i]] = tpoint[i];
				}
				tpoint.length = 0;
			}
		}
		calcAllPolyMidPoints();
	//	doBuild();
		ctrf = true;
	}
	resetRotationsAndTranslations();
}


void Seamless::resetRotations(){
	for(int k=partList.length; --k>=0;){
		partList[k]->rotation = Rotation();
	}
	catMatrix();
	catIMatrix();
	updateAllVertexInBuffers();//needed to stop dark normals but dont see y its needed?
}

void Seamless::resetRotationsAndTranslations(){
	for(int k=partList.length; --k>=0;){
		Art* part = partList[k];
		part->rotation = Rotation();
		part->translation = Vec3f();
	}
	catMatrix();
	catIMatrix();
	updateAllVertexInBuffers();//needed to stop dark normals but dont see y its needed?
}
int Seamless::getNextCtrlPoly(int poly){
	int len = ctrlPolyPoint[poly].len;
	poly += len;
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		if(midLen == 0){
			midLen++;
		}
		poly += midLen;
	}
	return poly;
}

void Seamless::getNextCtrlPoly(){
	if(ctrlPolyPoint.length == 0){
		return;
	}
	MultiSelect::ctrlPolySelList.length = 1;
	int selCtrlPoly = MultiSelect::ctrlPolySelList[0];

	if(selCtrlPoly >= ctrlPolyPoint.length){
		selCtrlPoly = 0;
	}
	selCtrlPoly = getNextCtrlPoly(selCtrlPoly);
	if(selCtrlPoly >= ctrlPolyPoint.length){
		selCtrlPoly = 0;
	}
	MultiSelect::ctrlPolySelList[0] = selCtrlPoly;
}

void Seamless::getPrevCtrlPoly(){
	if(ctrlPolyPoint.length == 0){
		return;
	}
	MultiSelect::ctrlPolySelList.length = 1;

	int targetPoly = MultiSelect::ctrlPolySelList[0];
	int prevPoly = MultiSelect::ctrlPolySelList[0];
	while(1){
		getNextCtrlPoly();
		if(targetPoly == MultiSelect::ctrlPolySelList[0]){
			MultiSelect::ctrlPolySelList[0] = prevPoly;
			return;
		}
		prevPoly = MultiSelect::ctrlPolySelList[0];
	}
}

void Seamless::deleteCtrlPoly(int poly){
	if(ctrlPolyPoint.length){
		int nextCtrlPoly = getNextCtrlPoly(poly);
		ctrlPolyPoint.deleteElement(poly, nextCtrlPoly - poly);
	}
}

void Seamless::deleteCtrlPoly(M_Int& deletePolyList){
	
	bubbleSort(deletePolyList);
	
	deleteLinksToPolys(deletePolyList);

	for(int q = deletePolyList.length; --q >= 0;){
		deleteCtrlPoly(deletePolyList[q]);
	}

}

void Seamless::calcAllPolyMidPoints(){
	for(int k = 0; k < cpNodeList.length; k++){
		cpNodeList[k]->pointsToDpoints();
	}

	for(int i = 0; i < ctrlPolyPoint.length;){
		calcMidPoints(i, ctrlPolyPoint);
		i = getNextCtrlPoly(i);
	}

	calcAllLinks();

	processAllPatches(cpNodeList);
}


void Seamless::redirectStitchPolyIndices(NurbsPatch* patch, const M_Int& lookup){
	for(int i = ctrlPolyPoint.length; --i >= 0;){
		if(ctrlPolyPoint[i].p == patch){
			ctrlPolyPoint[i].i = lookup[ctrlPolyPoint[i].i];
		}
	}
}

void Seamless::redirectStitchPolyIndices(NurbsPatch* patch, const M_ControlPoint& lookup){
	for(int i = ctrlPolyPoint.length; --i >= 0;){
		if(ctrlPolyPoint[i].p == patch){
			int idx = ctrlPolyPoint[i].i;
			ctrlPolyPoint[i].p = lookup[idx].p;
			ctrlPolyPoint[i].i = lookup[idx].i;
		}
	}
}

void Seamless::deleteLinksToPolys(M_Int& deletePolyList){
	if(deletePolyList.length == 0){
		return;
	}
	M_Int redirectCtrlPolyPoint;
	redirectCtrlPolyPoint.length = ctrlPolyPoint.length;

	for(int k = 0; k < deletePolyList.length; k++){
		int from = deletePolyList[k];
		int to = getNextCtrlPoly(from);
		for(int i = from; i < to; i++){
			redirectCtrlPolyPoint[i] = -1;
		}
	}

	for(int i = 0, cntr = -1; i < redirectCtrlPolyPoint.length; i++){
		if(redirectCtrlPolyPoint[i] >= 0){
			cntr++;
			redirectCtrlPolyPoint[i] = cntr;
		}
	}

	int d = 0;
	for(int s = 0; s < link.length; s++){
		int poly = link[s].poly;
		int i = redirectCtrlPolyPoint[poly];
		if(i == -1){
			continue;
		}
		link[d] = link[s];
		link[d++].poly = i;
	}
	link.length = d;
}

void Seamless::deleteLinksToNonExistentPatches(){
	int d = 0;
	for(int s = 0; s < link.length; s++){
		NurbsPatch* patch = link[s].patch;
		if(patch->plidx >= cpNodeList.length || cpNodeList[patch->plidx] != patch){
			continue;
		}
		link[d++] = link[s];
	}
	link.length = d;
}

bool Seamless::pointsExist(M_Int& deletePolyList, int poly, int& m, int len){
	for(int k = len; --k >= 0;){
		NurbsPatch* patch = ctrlPolyPoint[m++].p;
		if(patch->plidx >= cpNodeList.length || cpNodeList[patch->plidx] != patch){
			deletePolyList += poly;
			return false;
		}
	}
	return true;
}

void Seamless::deletePolysWithNonExistingPoints(){
	M_Int deletePolyList;
	int ctrlPolyPointLen = ctrlPolyPoint.length;

	for(int i = 0; i < ctrlPolyPointLen;){
		int poly = i;
		int m = poly;
		int len = ctrlPolyPoint[poly].len;
		if(false == pointsExist(deletePolyList, poly, m, len)){
			i = getNextCtrlPoly(poly);
			continue;
		}

		for(int o = len + 1; --o>=0;){
			int len = ctrlPolyPoint[m].len;
			if(len == 0){
				m++;
				continue;
			}
			if(false == pointsExist(deletePolyList, poly, m, len)){
				i = m = getNextCtrlPoly(poly);
				break;
			}
		}
		i = m;
	}

	deleteCtrlPoly(deletePolyList);
}

bool Seamless::inPolyCenter(NurbsPatch* patch, const int idx0, const int idx1, const int idx2){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		int centerLen = ctrlPolyPoint[poly].len;

		for(int k = centerLen; --k >= 0;){
			if(patch == ctrlPolyPoint[poly + k].p){
				int idxA = ctrlPolyPoint[poly + k].i;
				if(idxA == idx0 || idxA == idx1 || idxA == idx2){
					while(--k >= 0){
						if(patch == ctrlPolyPoint[poly + k].p){
							int idxB = ctrlPolyPoint[poly + k].i;
							if(idxB == idx0 || idxB == idx1 || idxB == idx2){
								return true;
							}
						}
					}
				}
			}
		}
		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}


void Seamless::whereYouSeeAddThis(const NurbsPatch* checkPatch, const int checkIndex, NurbsPatch* addPatch, int addIndex){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			for(int i = 0; i < midLen; i++){
				if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIndex){
					ctrlPolyPoint.insertElement(poly, CtrlPolyPoint(addPatch, addIndex, midLen + 1));
					return;
				}
			}

			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;
		for(int i = 0; i < centerLen; i++){
			if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIndex){
				ctrlPolyPoint.insertElement(poly, CtrlPolyPoint(addPatch, addIndex, centerLen + 1));
				return;
			}
		}

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}

void findStitchPolyAndAddForCenter(NurbsPatch* patch, int addIdx, int centerGuideIdx, int nextCorner, M_CtrlPolyPoint& ctrlPolyPoint, int poly){
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		if(ctrlPolyPoint[poly + k].p == patch && ctrlPolyPoint[poly + k].i == centerGuideIdx){
			k = getCorner(k, len + nextCorner, len);
			poly += len;
			for(int j = 0; ; j++){
				int midLen = ctrlPolyPoint[poly].len;

				if(j == k){
					if(midLen == 0){
						ctrlPolyPoint[poly] = CtrlPolyPoint(patch, addIdx, 1);
					}
					else{
						ctrlPolyPoint.insertElement(poly, CtrlPolyPoint(patch, addIdx, midLen + 1));
					}
					return;
				}

				if(midLen == 0){
					midLen++;
				}
				poly += midLen;
			}
		}
	}
}

void Seamless::findStitchPolyAndAdd(const NurbsPatch* checkPatch, const int checkIdx, NurbsPatch* addPatchA, int addIdxA, int centerGuideIdxA, int nextCornerA, NurbsPatch* addPatchB, int addIdxB, int centerGuideIdxB, int nextCornerB){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		int beginPoly = poly;
		poly += len;
		int remCtrlPoly = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			for(int i = 0; i < midLen; i++){
				if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIdx){
					if(k == 1){
						poly = remCtrlPoly;
					}
					else{
						poly += midLen;
						midLen = ctrlPolyPoint[poly].len;
						if(midLen == 0){
							midLen++;
						}
						poly += midLen;
					}
					midLen = ctrlPolyPoint[poly].len;

					if(midLen == 0){
						ctrlPolyPoint[poly].p = addPatchA;
						ctrlPolyPoint[poly].i = addIdxA;
					}
					else{
						ctrlPolyPoint.insertElement(poly, CtrlPolyPoint(addPatchA, addIdxA, 0));
					}
					ctrlPolyPoint.insertElement(poly, CtrlPolyPoint(addPatchB, addIdxB, midLen + 2));
					return;
				}
			}

			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;

		for(int i = 0; i < centerLen; i++){
			if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIdx){

				findStitchPolyAndAddForCenter(addPatchA, addIdxA, centerGuideIdxA, nextCornerA, ctrlPolyPoint, beginPoly);
				findStitchPolyAndAddForCenter(addPatchB, addIdxB, centerGuideIdxB, nextCornerB, ctrlPolyPoint, beginPoly);

				return;
			}
		}

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}
bool Seamless::initTugStitchPoly(const bool& perspective){
	bool hit = false;
	for(int poly = 0; poly < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[poly].len;
		for(int k = 0; k < len; k++){
			NurbsPatch* patch = ctrlPolyPoint[poly + k].p;
			if(patch->cage){
				int idx = ctrlPolyPoint[poly + k].i;
				int uDim = patch->dim.x;
				int vDim = patch->dim.y;
				int u = idx % uDim;
				int v = idx / uDim;
				Vec3f pnt = g_viewMatrix * patch->dpoint[idx];
				hit = patch->checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, idx, g_viewMatrix, patch->dpoint.refValue());
			}
		}
		poly = getNextCtrlPoly(poly);
	}
	return hit;
}

Matrix Seamless::getSmlsViewMatrix(){
	return scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * g_scene->getViewMatrix();
}

Matrix Seamless::getSmlsIViewMatrix(){
	return g_scene->getIViewMatrix() * translationMatrix(-translation) * rotationMatrix(-rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z));
}

void Seamless::createSelContigList(M_<NurbsPatch*>& contigList){
	contigList.length = pointTotal;

	for(int k = MultiSelect::pointList.length; --k >= 0;){
		NurbsPatch* patch = MultiSelect::pointList[k].p;
		int i = MultiSelect::pointList[k].i;
		contigList[patch->fromPoint + i] = patch;
	}
}

void Seamless::clearObjects(){
	for(int k = 0; k < cpNodeList.length; k++){
		cpNodeList[k]->object = false;
	}
}
void Seamless::addExtraCoordsRender(M_Int& coordIndex){//and populate matrixIndex
	if(partList.length == 0) return;
	Effect* effect=partList[0]->workEffect;
	if(effect == 0) return;
	static M_Int lookUpCoord;
	lookUpCoord.length = effect->getEffectLength();
	{
		for(int i = lookUpCoord.length; --i>=0; ){
			lookUpCoord[i] = -1;
		}
	}
	{
		for(int mi = 0, j = 0, k = 0; k < partList.length; k++,mi++){
			Art* part=partList[k];
			part->rc.length = 0;
			M_Int& matrixIndex = part->matrixIndex;
			matrixIndex.length = part->coord.length;
			const M_Int& coordGroup = part->coordGroup;
			{
				int j = 0;
				for(int g=0; g<coordGroup.length; g++, mi++ ){
					for(int i = 0; i < coordGroup[g] && j < matrixIndex.length; i++){
						matrixIndex[j++] = mi;
					}
				}
				while(j<matrixIndex.length){
					matrixIndex[j++] = mi;
				}

			}
		}
	}
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(effect)){
		const M_Color3f& color = colorEffect->color;

		for(int j = 0, k = 0; k<partList.length; k++){
			Art* part = partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length * 3;
			for(int o = 3, i = 0; i<triLen; i++){
				int linTci = triangle.refTexIndex(i);
				int ci = triangle.refIndex(i);
				Art* cornerPart = triangle.refPart(i);
				M_Int& localCoordLookUp = cornerPart->partTree;
				if( lookUpCoord[linTci]==-1){
					if( localCoordLookUp[ci] == -1){
						localCoordLookUp[ci] = linTci;
						cornerPart->color[ci] = color[linTci];
						lookUpCoord[linTci] = ci;
						coordIndex[j++] = ci;
					}
					else {
						coordIndex[j++] = lookUpCoord[linTci] = cornerPart->coord.length;
						cornerPart->color += color[linTci];
						cornerPart->coord.refValue() += cornerPart->coord[ci];
						cornerPart->normal.refValue() += cornerPart->normal[ci];
						cornerPart->matrixIndex += cornerPart->matrixIndex[ci];
						cornerPart->rc += ci;
					}
				}
				else {
					coordIndex[j++] = lookUpCoord[linTci];
				}
				if(--o==0){
					o = 3;
					j++;//coordIndex[j++]=-1;
				}
			}
		}
	}
	else if(TextureEffect* texEffect=dynamic_cast<TextureEffect*>(effect)){
		M_Vec2f& texCoord = texEffect->texCoord.refValue();

		for(int j=0, k=0; k<partList.length; k++){
			Art* part = partList[k];
			M_Triangle& triangle = part->triangle;
			int triLen=triangle.length * 3;
			for(int o=3, i = 0; i < triLen; i++){
				int linTci = triangle.refTexIndex(i);
				int ci = triangle.refIndex(i);
				Art* cornerPart = triangle.refPart(i);
				M_Int& localCoordLookUp = cornerPart->partTree;
				if(lookUpCoord[linTci] == -1){
					if(localCoordLookUp[ci] == -1){
						localCoordLookUp[ci] = linTci;
						cornerPart->texCoord[ci] = texCoord[linTci];
						lookUpCoord[linTci] = ci;
						coordIndex[j++] = ci;
					}
					else {
						coordIndex[j++] = lookUpCoord[linTci] = cornerPart->coord.length;
						cornerPart->texCoord += texCoord[linTci];
						cornerPart->coord.refValue() += cornerPart->coord[ci];
						cornerPart->normal.refValue() += cornerPart->normal[ci];
						cornerPart->matrixIndex += cornerPart->matrixIndex[ci];
						cornerPart->rc += ci;
					}
				}
				else {
					coordIndex[j++] = lookUpCoord[linTci];
				}
				if(--o==0){
					o = 3;
					j++;//coordIndex[j++]=-1;
				}
			}
		}
	}
	else if(MonoColorEffect* monoColorEffect=dynamic_cast<MonoColorEffect*>(effect)){
		const M_Color3f& color = colorEffect->color;

		for(int j = 0, k = 0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle = part->triangle;
			int triLen = triangle.length * 3;
			for(int o = 3, i = 0; i < triLen; i++){
				int linTci = triangle.refTexIndex(i);
				int ci = triangle.refIndex(i);
				Art* cornerPart = triangle.refPart(i);
				M_Int& localCoordLookUp = cornerPart->partTree;
				if( lookUpCoord[linTci] == -1){
					if( localCoordLookUp[ci] == -1){
						localCoordLookUp[ci] = linTci;
						cornerPart->color[ci] = color[linTci];
						lookUpCoord[linTci] = ci;
						coordIndex[j++] = ci;
					}
					else {
						coordIndex[j++] = lookUpCoord[linTci]=cornerPart->coord.length;
						cornerPart->color += color[linTci];
						cornerPart->coord.refValue() += cornerPart->coord[ci];
						cornerPart->normal.refValue() += cornerPart->normal[ci];
						cornerPart->matrixIndex += cornerPart->matrixIndex[ci];
						cornerPart->rc += ci;
					}
				}
				else {
					coordIndex[j++] = lookUpCoord[linTci];
				}
				if(--o == 0){
					o = 3;
					j++;//coordIndex[j++]=-1;
				}
			}
		}
	}
	{
		int linCoordLen=0;
		for(int k=0; k<partList.length; k++){
			Art* part=partList[k];
			part->fromCoord=linCoordLen;
			linCoordLen+=part->coord.length;
		}
	}
	{
		for(int j=0, k=0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length*3;
			for(int i=0; i<triLen; ){
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
				j++;
			}
		}
	}
}

void Seamless::addExtraCoordsRenderForGeomSource(M_Int& coordIndex){
	if(partList.length==0) return;
	Effect* effect=partList[0]->workEffect;
	if(effect==0) return;
	static M_Int lookUpCoord;
	lookUpCoord.length=effect->getEffectLength();
	{
		for(int i=lookUpCoord.length; --i>=0; ){
			lookUpCoord[i]=-1;
		}
	}
	{
		for(int mi=0, j=0, k=0; k<partList.length; k++,mi++){
			Art* part=partList[k];
			part->rc.length=0;
			M_Int& matrixIndex=part->matrixIndex;
			matrixIndex.length=part->coord.length;
			const M_Int& coordGroup=part->coordGroup;
			{
				int j=0;
				for(int g=0; g<coordGroup.length; g++, mi++ ){
					for(int i=0; i<coordGroup[g]&&j<matrixIndex.length; i++){
						matrixIndex[j++]=mi;
					}
				}
				while(j<matrixIndex.length){
					matrixIndex[j++]=mi;
				}

			}
		}
	}
	if(ColorEffect* colorEffect=dynamic_cast<ColorEffect*>(effect)){
		const M_Color3f& color=colorEffect->color;

		for(int j=0, k=0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length*3;
			for(int i=0; i<triLen; i++){
				int linTci=triangle.refTexIndex(i);
				int ci=triangle.refIndex(i);
				Art* cornerPart=triangle.refPart(i);
				M_Int& localCoordLookUp=cornerPart->partTree;
				if( lookUpCoord[linTci]==-1){
					if( localCoordLookUp[ci]==-1){
						localCoordLookUp[ci]=linTci;
						cornerPart->color[ci]=color[linTci];
						lookUpCoord[linTci]=ci;
						coordIndex[j++]=ci;
					}
					else {
						coordIndex[j++]=lookUpCoord[linTci]=cornerPart->coord.length;
						cornerPart->color+=color[linTci];
						cornerPart->coord.refValue()+=cornerPart->coord[ci];
						cornerPart->normal.refValue()+=cornerPart->normal[ci];
						cornerPart->matrixIndex+=cornerPart->matrixIndex[ci];
						cornerPart->rc+=ci;
					}
				}
				else {
					coordIndex[j++]=lookUpCoord[linTci];
				}
			}
		}
	}
	else if(TextureEffect* texEffect=dynamic_cast<TextureEffect*>(effect)){
		M_Vec2f& texCoord=texEffect->texCoord.refValue();

		for(int j=0, k=0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length*3;
			for(int i=0; i<triLen; i++){
				int linTci=triangle.refTexIndex(i);
				int ci=triangle.refIndex(i);
				Art* cornerPart=triangle.refPart(i);
				M_Int& localCoordLookUp=cornerPart->partTree;
				if( lookUpCoord[linTci]==-1){
					if( localCoordLookUp[ci]==-1){
						localCoordLookUp[ci]=linTci;
						cornerPart->texCoord[ci]=texCoord[linTci];
						lookUpCoord[linTci]=ci;
						coordIndex[j++]=ci;
					}
					else {
						coordIndex[j++]=lookUpCoord[linTci]=cornerPart->coord.length;
						cornerPart->texCoord+=texCoord[linTci];
						cornerPart->coord.refValue()+=cornerPart->coord[ci];
						cornerPart->normal.refValue()+=cornerPart->normal[ci];
						cornerPart->matrixIndex+=cornerPart->matrixIndex[ci];
						cornerPart->rc+=ci;
					}
				}
				else {
					coordIndex[j++]=lookUpCoord[linTci];
				}
			}
		}
	}
	if(MonoColorEffect* colorEffect=dynamic_cast<MonoColorEffect*>(effect)){
		const M_Color3f& color=colorEffect->color;

		for(int j=0, k=0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length*3;
			for(int i=0; i<triLen; i++){
				int linTci=triangle.refTexIndex(i);
				int ci=triangle.refIndex(i);
				Art* cornerPart=triangle.refPart(i);
				M_Int& localCoordLookUp=cornerPart->partTree;
				if( lookUpCoord[linTci]==-1){
					if( localCoordLookUp[ci]==-1){
						localCoordLookUp[ci]=linTci;
						cornerPart->color[ci]=color[linTci];
						lookUpCoord[linTci]=ci;
						coordIndex[j++]=ci;
					}
					else {
						coordIndex[j++]=lookUpCoord[linTci]=cornerPart->coord.length;
						cornerPart->color+=color[linTci];
						cornerPart->coord.refValue()+=cornerPart->coord[ci];
						cornerPart->normal.refValue()+=cornerPart->normal[ci];
						cornerPart->matrixIndex+=cornerPart->matrixIndex[ci];
						cornerPart->rc+=ci;
					}
				}
				else {
					coordIndex[j++]=lookUpCoord[linTci];
				}
			}
		}
	}
	{
		int linCoordLen=0;
		for(int k=0; k<partList.length; k++){
			Art* part=partList[k];
			part->fromCoord=linCoordLen;
			linCoordLen+=part->coord.length;
		}
	}
	{
		for(int j=0, k=0; k<partList.length; k++){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			int triLen=triangle.length*3;
			for(int i=0; i<triLen; ){
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
				coordIndex[j++]+=triangle.refPart(i++)->fromCoord;
			}
		}
	}
}
void Seamless::genCage(){
	g_pointLookup.length = 0;
	g_pointLookup.length = cpNodeList.length;
	M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
	M_Color3f& color = dynamic_cast<Color*>(g_nurbsIlsColor)->color;

	point.length = 0;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();
	coordIndex.length = 0;

	for(int k = 0; k < cpNodeList.length; k++){
		Build* bld = cpNodeList[k]->container;
		if(dynamic_cast<NurbsPatch*>(bld) == 0){
			k += bld->numOfPatches() - 1;
			continue;
		}
		NurbsPatch* patch = bld->getPatch();

		if(dynamic_cast<Strand*>(patch)){
			if((g_scene->mode & SHOW_STRAND_CTRL_LINE_MODE) == false){
				continue;
			}
		}
		
		g_pointLookup[patch->plidx] = point.length;

		patch->genCage(color);
	}

	addCageControlPolysColorCode();


	color.length = point.length;
	for(int i = color.length; --i >= 0;){
		color[i] = Color3f(0,1,1);
	}


	g_vcBezierBase->rotation=Rotation();

	g_nurbsIlsShape->convertToRenderFormat();

	g_insertPatch.beginCoordIndexVisualClue = coordIndex.length;
	g_insertPatch.beginCoordVisualClue = color.length;
}
void Seamless::genStraightCage(const M_Bool& m_pointIsVisible){
	M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;

	point.length = 0;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();
	coordIndex.length = 0;

	g_nurbsIlsCoord->point.refValue().length = pointTotal;
	M_Color3f& color = g_nurbsIlsColor->color.refValue();
	color.length = pointTotal;
	genVisibleMask();


	if(g_transformCP->selectObject){
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			patch->genStraightCage(color, m_pointIsVisible);
		}
	}
	else{
		for(int k = 0; k < cpNodeList.length; k++){
			Build* bld = cpNodeList[k]->container;
			if(dynamic_cast<NurbsPatch*>(bld) == 0){
				k += bld->numOfPatches() - 1;
				continue;
			}
			NurbsPatch* patch = bld->getPatch();

			if(dynamic_cast<Strand*>(patch)){
				if((g_scene->mode & SHOW_STRAND_CTRL_LINE_MODE) == false){
					continue;
				}
			}

			patch->genStraightCage(color, m_pointIsVisible);
		}
	}
	if(g_showPatchBorderMode == false){
		if((g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE) == false){
			colorLinkPoints();
			addStraightCageControlPolysColorCode(m_pointIsVisible);

		}
	}
	//int d = 0;
	//int pointCount = 0;
	//for(int s = 0 ; s < coordIndex.length; ){
	//	int idx = coordIndex[s++];
	//	if(idx == -1){
	//		if(pointCount == 1){
	//			d--;
	//		}
	//		pointCount = 0;
	//		coordIndex[d++] = idx;
	//		continue;
	//	}
	//	Color3f clr = color[idx];
	//	if(clr == Color3f(0,1,1)){
	//		coordIndex[d++] = idx;
	//		pointCount++;
	//	}
	//}
	//coordIndex.length = d;


	g_vcBezierBase->rotation=Rotation();

	g_nurbsIls->convertToRenderFormat(g_nurbsIlsShape->mesh, g_nurbsIlsShape->appearance);
	



	g_insertPatch.beginCoordIndexVisualClue = coordIndex.length;
	g_insertPatch.beginCoordVisualClue = color.length;
}
void Seamless::addStraightCageControlPolysColorCode(const M_Bool& m_pointIsVisible){
	M_Vec3f& point=dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
	M_Color3f& color = dynamic_cast<Color*>(g_nurbsIlsColor)->color;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();

	int ctrlPolyPointLen = ctrlPolyPoint.length;
	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		for(int i = 0; i < ctrlPolyPointLen;){
			int ctrlPoly = i;
			int len = ctrlPolyPoint[ctrlPoly].len;
			int mark = coordIndex.length;
			bool flag = false;
			int m = i + len;
			for(int k = 0; k < len;){
				NurbsPatch* patchA = ctrlPolyPoint[i].p;
				int iA  = ctrlPolyPoint[i].i;
				k++;
				if(m_pointIsVisible[patchA->fromPoint + iA]){
					int i2 = i + 1;
					if(k >= len){
						i2 = ctrlPoly;
					}
					NurbsPatch* patchB = ctrlPolyPoint[i2].p;
					int iB  = ctrlPolyPoint[i2].i;
					i++;
					if(m_pointIsVisible[patchB->fromPoint + iB] == false){
						continue;
					}
					flag |= patchA->cage;
					flag |= patchB->cage;
					coordIndex += patchA->fromPoint + iA;
					coordIndex += patchB->fromPoint + iB;
					i++;
					k++;
					for(; k <= len; ){
						if(k >= len){
							i = ctrlPoly;
						}
						patchA = ctrlPolyPoint[i].p;
						iA  = ctrlPolyPoint[i].i;
						k++;
						if(m_pointIsVisible[patchA->fromPoint + iA]){
							i++;
							flag |= patchA->cage;
							coordIndex += patchA->fromPoint + iA;
						}
						else{
							break;
						}
					}
					coordIndex += -1;
				}
			}
			if(flag == false){
				coordIndex.length = mark;
				i = getNextCtrlPoly(ctrlPoly);
				continue;
			}
			for(int o = len + 1; --o>=0;){
				int midLen = ctrlPolyPoint[m].len;
				if(midLen == 0){
					m++;
					continue;
				}
				for(int j = midLen; --j>=0;){
					color[ctrlPolyPoint[m].p->fromPoint + ctrlPolyPoint[m].i] = Color3f(0,1,0);
					m++;
				}
			}
			i = m;
		}
		//coordIndex += ctrlPolyPoint[ctrlPoly].p->fromPoint + ctrlPolyPoint[ctrlPoly].i;
		//coordIndex += -1;
	}
	else{
		for(int i = 0; i < ctrlPolyPointLen;){
			int ctrlPoly = i;
			int len = ctrlPolyPoint[ctrlPoly].len;
			int mark = coordIndex.length;
			bool flag = false;
			int m = i + len;
			for(int k = 0; k < len;){
				NurbsPatch* patchA = ctrlPolyPoint[i].p;
				int iA  = ctrlPolyPoint[i].i;
				k++;
				flag |= patchA->cage;
				coordIndex += patchA->fromPoint + iA;
				i++;
			}
			NurbsPatch* patchA = ctrlPolyPoint[ctrlPoly].p;
			int iA  = ctrlPolyPoint[ctrlPoly].i;
			flag |= patchA->cage;
			coordIndex += patchA->fromPoint + iA;
			coordIndex += -1;

			if(flag == false){
				coordIndex.length = mark;
				i = getNextCtrlPoly(ctrlPoly);
				continue;
			}
			for(int o = len + 1; --o>=0;){
				int midLen = ctrlPolyPoint[m].len;
				if(midLen == 0){
					m++;
					continue;
				}
				for(int j = midLen; --j>=0;){
					color[ctrlPolyPoint[m].p->fromPoint + ctrlPolyPoint[m].i] = Color3f(0,1,0);
					m++;
				}
			}
			i = m;
		}
	}

	if(g_scene->mode & STITCH_POLY_MODE && ctrlPolyPoint.length){
		genTransparentCtrlPolys(g_vcNurbsCtrlPolyPart, point);
	}
}

void NurbsPatch::cpwsToPoints(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	int uDim = dim.x;
	int vDim = dim.y;
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	const M_Int& cpIndex = this->cpIndex;

	point.length = vDim * uDim;
	weight.length = vDim * uDim;

	for(int i = cpIndex.length; --i>=0;){
		int cpi = cpIndex[i];
		point[i] = cp[cpi];
		weight[i] = cw[cpi];
	}
	processFields();
}

void Seamless::centralCPsToPatches(){
	for(int k = cpNodeList.length; --k>=0;){
		NurbsPatch* patch = cpNodeList[k];
		patch->cpwsToPoints();
	}
}

void NurbsPatch::setPointAndCp(int i, Vec3f vec){
	M_Vec3f& point = this->point.refValue();
	M_Vec3f& cp = smls->cp.refValue();
	int cpi = cpIndex[i];
	cp[cpi] = point[i] = vec;
}

void NurbsPatch::patchToCentralCPs(){ //for debugging only
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();
	int uDim = dim.x;
	int vDim = dim.y;
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();

	for(int i = cpIndex.length; --i>=0;){
		setPointAndCp(i, point[i]);
		//int cpi = cpIndex[i];
		//cp[cpi] = point[i];
		//cw[cpi] = weight[i];
	}
}

void Seamless::patchesToCentralCPs(){ //for debugging only
	M_Vec3f& cp = this->cp.refValue();
	M_Float& cw = this->cw.refValue();
	for(int k = cpNodeList.length; --k>=0;){
		NurbsPatch* patch = cpNodeList[k];
		patch->patchToCentralCPs();
/*


		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Vec3f& point = patch->point.refValue();
		M_Float& weight = patch->weight.refValue();
		const M_Int& cpIndex = patch->cpIndex;

		for(int i = cpIndex.length; --i>=0;){
			int cpi = cpIndex[i];
			cp[cpi] = point[i];
			cw[cpi] = weight[i];
		}
		*/
	}
}

void NurbsPatch::createCpFromPatchPoints(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	int u = dim.x;
	int v = dim.y;
	int len = v * u;
	M_Int& cpIndex = this->cpIndex.refValue();

	int fromPoint = cp.length;


	point.refValue().length = u * v;
	weight.refValue().length = u * v;

	cp += point;
	cw += weight;

	cpIndex.length = len;

	for(int i = 0; i < len; i++){
		cpIndex[i] = fromPoint + i;
	}
}

void Seamless::createCpFromPatchPoints(){
	cp.refValue().length = 0;
	cw.refValue().length = 0;

	for(int k = 0; k < cpNodeList.length;){
		Build* container = cpNodeList[k]->container;
		container->createCpFromPatchPoints();
		k += container->numOfPatches();
		//NurbsPatch* patch = cpNodeList[k];
		//patch->createCpFromPatchPoints();
		//k += patch->numOfPatches();
	}
}
void Seamless::cleanCps(){
	M_Vec3f& cp = this->cp.refValue();
	M_Float& cw = this->cw.refValue();
	M_Int lookup;
	lookup.length = cp.length;

	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		const M_Int& cpIndex = patch->cpIndex.refValue();
		for(int i = cpIndex.length; --i >= 0;){
			lookup[cpIndex[i]] = 1;
		}
	}
	int d = 0;
	for(int s = 0; s < cp.length;){
		if(lookup[s]){
			lookup[s] = d;
			cp[d] = cp[s];
			cw[d] = cw[s];
			d++;
		}
		s++;
	}
	cp.length = d;
	cw.length = d;

	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		M_Int& cpIndex = patch->cpIndex.refValue();
		for(int i = cpIndex.length; --i >= 0;){
			cpIndex[i] = lookup[cpIndex[i]];
		}
	}
}
